@@ -5,7 +5,8 @@ from biblemate.ui.selection_dialog import TerminalModeDialogs
 from biblemate import config, AGENTMAKE_CONFIG, OLLAMA_NOT_FOUND
 from biblemate.core.bible_db import BibleVectorDatabase
 from pathlib import Path
-import asyncio, re, os, subprocess, click, shutil
+import asyncio, re, os, subprocess, click, shutil, pprint
+from copy import deepcopy
 from alive_progress import alive_bar
 from fastmcp import Client
 from agentmake import agentmake, getOpenCommand, getDictionaryOutput, edit_configurations, writeTextFile, getCurrentDateTime, AGENTMAKE_USER_DIR, USER_OS, DEVELOPER_MODE, DEFAULT_AI_BACKEND
@@ -65,6 +66,7 @@ async def initialize_app(client):
     available_tools = list(tools.keys())
     if "get_direct_text_response" not in available_tools:
         available_tools.insert(0, "get_direct_text_response")
+    master_available_tools = deepcopy(available_tools)
 
     tool_descriptions = ""
     if "get_direct_text_response" not in tools:
@@ -100,7 +102,7 @@ Get a static text-based response directly from a text-based AI model without usi
         }
         prompts_schema[p.name] = schema
     
-    return tools, tools_schema, available_tools, tool_descriptions, prompts, prompts_schema
+    return tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema
 
 def backup_conversation(console, messages, master_plan):
     """Backs up the current conversation to the user's directory."""
@@ -109,7 +111,7 @@ def backup_conversation(console, messages, master_plan):
     Path(storagePath).mkdir(parents=True, exist_ok=True)
     # Save full conversation
     conversation_file = os.path.join(storagePath, "conversation.py")
-    writeTextFile(conversation_file, str(messages))
+    writeTextFile(conversation_file, pprint.pformat(messages))
     # Save master plan
     writeTextFile(os.path.join(storagePath, "master_plan.md"), master_plan)
     # Save markdown
@@ -138,7 +140,7 @@ async def main_async():
 
     APP_START = True
     DEFAULT_SYSTEM = "You are BibleMate AI, an autonomous agent designed to assist users with their Bible study."
-    DEFAULT_MESSAGES = [{"role": "user", "content": "Hello!"}, {"role": "assistant", "content": "Hello! I'm BibleMate AI, your personal assistant for Bible study. How can I help you today?"}] # set a tone for bible study; it is userful when auto system is used.
+    DEFAULT_MESSAGES = [{"role": "system", "content": DEFAULT_SYSTEM}, {"role": "user", "content": "Hello!"}, {"role": "assistant", "content": "Hello! I'm BibleMate AI, your personal assistant for Bible study. How can I help you today?"}] # set a tone for bible study; it is userful when auto system is used.
 
     console = Console(record=True)
     console.clear()
@@ -146,7 +148,7 @@ async def main_async():
     dialogs = TerminalModeDialogs(None)
 
     async with client:
-        tools, tools_schema, available_tools, tool_descriptions, prompts, prompts_schema = await initialize_app(client)
+        tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema = await initialize_app(client)
         
         available_tools_pattern = "|".join(available_tools)
         prompt_list = [f"/{p}" for p in prompts.keys()]
@@ -155,7 +157,7 @@ async def main_async():
 
         user_request = ""
         master_plan = ""
-        messages = DEFAULT_MESSAGES # set the tone
+        messages = deepcopy(DEFAULT_MESSAGES) # set the tone
 
         while not user_request == ".quit":
 
@@ -286,6 +288,14 @@ async def main_async():
                     console.print(Markdown("Viist https://github.com/eliranwong/biblemate for help."))
                     console.rule()
                 elif user_request == ".tools":
+                    enabled_tools = await dialogs.getMultipleSelection(
+                        default_values=available_tools,
+                        options=master_available_tools,
+                        title="Tool Options",
+                        text="Select tools to enable:"
+                    )
+                    if enabled_tools is not None:
+                        available_tools = enabled_tools
                     console.rule()
                     tools_descriptions = [f"- `{name}`: {description}" for name, description in tools.items()]
                     console.print(Markdown("## Available Tools\n\n"+"\n".join(tools_descriptions)))
@@ -350,7 +360,7 @@ async def main_async():
                 if user_request == ".new":
                     user_request = ""
                     master_plan = ""
-                    messages = DEFAULT_MESSAGES
+                    messages = deepcopy(DEFAULT_MESSAGES)
                     console.clear()
                     console.print(get_banner())
                 continue
@@ -370,7 +380,9 @@ async def main_async():
                 async def refine_custom_plan():
                     nonlocal messages, user_request, master_plan
                     # Summarize user request in one-sentence instruction
-                    user_request = agentmake(master_plan, tool="biblemate/summarize_task_instruction", **AGENTMAKE_CONFIG)[-1].get("content", "").strip()[15:-4]
+                    user_request = agentmake(master_plan, tool="biblemate/summarize_task_instruction", **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
+                    if "```" in user_request:
+                        user_request = re.sub(r"^.*?(```instruction|```)(.+?)```.*?$", r"\2", user_request, flags=re.DOTALL).strip()
                 await thinking(refine_custom_plan)
                 # display info
                 console.print(Markdown(f"# User Request\n\n{user_request}\n\n# Master plan\n\n{master_plan}"))
@@ -380,7 +392,9 @@ async def main_async():
                 async def run_prompt_engineering():
                     nonlocal user_request
                     try:
-                        user_request = agentmake(messages if messages else user_request, follow_up_prompt=user_request if messages else None, tool="improve_prompt", **AGENTMAKE_CONFIG)[-1].get("content", "").strip()[20:-4]
+                        user_request = agentmake(messages if messages else user_request, follow_up_prompt=user_request if messages else None, tool="improve_prompt", **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
+                        if "```" in user_request:
+                            user_request = re.sub(r"^.*?(```improved_version|```)(.+?)```.*?$", r"\2", user_request, flags=re.DOTALL).strip()
                     except:
                         user_request = agentmake(messages if messages else user_request, follow_up_prompt=user_request if messages else None, system="improve_prompt_2")[-1].get("content", "").strip()
                         user_request = re.sub(r"^.*?(```improved_prompt|```)(.+?)```.*?$", r"\2", user_request, flags=re.DOTALL).strip()
@@ -474,6 +488,7 @@ Available tools are: {available_tools}.
                     # partner mode
                     if config.agent_mode == False:
                         console.rule()
+                        console.print(Markdown("# Review & Confirm"))
                         console.print("Please review and confirm the master plan, or make any changes you need:", justify="center")
                         console.rule()
                         master_plan_edit = await getInput(default_entry=master_plan)
@@ -491,22 +506,24 @@ Available tools are: {available_tools}.
                     console.print(Markdown(master_plan), "\n\n")
 
             # Step suggestion system message
-            system_suggestion = get_system_suggestion(master_plan)
+            system_progress = get_system_progress(original_request=user_request, master_plan=master_plan)
+            system_make_suggestion = get_system_make_suggestion(original_request=user_request, master_plan=master_plan)
 
             # Tool selection systemm message
             system_tool_selection = get_system_tool_selection(available_tools, tool_descriptions)
 
             # Get the first suggestion
-            next_suggestion = ""
-            async def get_first_suggestion():
-                nonlocal next_suggestion
-                console.print(Markdown("## Suggestion [1]"), "\n")
-                next_suggestion = agentmake(user_request, system=system_suggestion, **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
-            await thinking(get_first_suggestion)
-            console.print(Markdown(next_suggestion), "\n\n")
+            next_suggestion = "START"
 
             step = 1
-            while not ("DONE" in next_suggestion or re.sub("^[^A-Za-z]*?([A-Za-z]+?)[^A-Za-z]*?$", r"\1", next_suggestion).upper() == "DONE"):
+            while not ("STOP" in next_suggestion or re.sub("^[^A-Za-z]*?([A-Za-z]+?)[^A-Za-z]*?$", r"\1", next_suggestion).upper() == "STOP"):
+
+                async def make_next_suggestion():
+                    nonlocal next_suggestion, system_make_suggestion, messages, step
+                    console.print(Markdown(f"## Suggestion [{step}]"), "\n")
+                    next_suggestion = agentmake(user_request if next_suggestion == "START" else [{"role": "system", "content": system_make_suggestion}]+messages[len(DEFAULT_MESSAGES):], system=system_make_suggestion, follow_up_prompt=None if next_suggestion == "START" else "Please provide me with the next step suggestion, based on the action plan.", **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
+                await thinking(make_next_suggestion)
+                console.print(Markdown(next_suggestion), "\n\n")
 
                 # Get tool suggestion for the next iteration
                 suggested_tools = []
@@ -552,13 +569,13 @@ Available tools are: {available_tools}.
                 # partner mode
                 if config.agent_mode == False:
                     console.rule()
+                    console.print(Markdown("# Review & Confirm"))
                     console.print("Please review and confirm the next step, or make any changes you need:")
                     console.rule()
                     next_step_edit = await getInput(default_entry=next_step)
                     if not next_step_edit or next_step_edit == ".quit":
                         console.rule()
                         console.print("I've stopped processing for you.")
-                        #console.rule()
                         break
                     else:
                         next_step = next_step_edit
@@ -582,12 +599,9 @@ Available tools are: {available_tools}.
 
                 # Get the next suggestion
                 async def get_next_suggestion():
-                    nonlocal next_suggestion, messages, system_suggestion
-                    console.print(Markdown(f"## Suggestion [{step}]"), "\n")
-                    next_suggestion = agentmake(messages, system=system_suggestion, follow_up_prompt="Please provide me with the next suggestion.", **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
-                await thinking(get_next_suggestion)
-                #print()
-                console.print(Markdown(next_suggestion), "\n")
+                    nonlocal next_suggestion, messages, system_progress
+                    next_suggestion = agentmake([{"role": "system", "content": system_progress}]+messages[len(DEFAULT_MESSAGES):], system=system_progress, follow_up_prompt="Please decide either to `CONTINUE` or `STOP` the process.", **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
+                await thinking(get_next_suggestion, description="Checking the progress ...")
             
             if messages[-1].get("role") == "user":
                 messages.append({"role": "assistant", "content": next_suggestion})
